#
# $Id: ChartTemplate.py,v 1.11 2004/10/24 10:30:35 admin Exp $
#
# Copyright (c) 2003, 2004 Dirk Datzert
# All rights reserved. Written by Dirk Datzert <dirk@datzert.de>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__version__='$Revision: 1.11 $'[11:-2]

import os
from string import lower
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens, \
                                      manage_users, \
                                      view


from RenderableObject import *
from Charts import *
from Labels import *
from Legends import *

from reportlab.graphics.charts.textlabels import Label
from reportlab.graphics.shapes import *

try:
	# Samples
	from reportlab.graphics.samples.bubble import *
	from reportlab.graphics.samples.clustered_bar import *
	from reportlab.graphics.samples.clustered_column import *
	from reportlab.graphics.samples.exploded_pie import *
	from reportlab.graphics.samples.filled_radar import *
	from reportlab.graphics.samples.line_chart import *
	from reportlab.graphics.samples.linechart_with_markers import *
	from reportlab.graphics.samples.radar import *
	from reportlab.graphics.samples.scatter_lines_markers import *
	from reportlab.graphics.samples.scatter_lines import *
	from reportlab.graphics.samples.scatter import *
	from reportlab.graphics.samples.simple_pie import *
	from reportlab.graphics.samples.stacked_bar import *
	from reportlab.graphics.samples.stacked_column import *

	reportlab_graphics_samples_installed = 1

except:

	reportlab_graphics_samples_installed = 0


_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

manage_addChartTemplateForm = \
	DTMLFile( 'manage_addChartTemplateForm', _dtmldir )

ADD_CHART_TEMPLATE_PERMISSION = 'Add ChartTemplate'


class ChartTemplate( RenderableObject ):
	""" 
		ChartTemplate

	"""

	meta_type = 'ChartTemplate'

	_meta_types = []

	_properties = RenderableObject._properties \
		+ ( { 'id': 'title', 'type': 'string', 'mode': 'w' }, )

    #################################################################
    #
    # Setting up all ZMI management screens and default login pages
    #
    #################################################################

	manage_options = RenderableObject._manage_options + \
        (
        {'label' : 'Test',	'action' : 'manage_test', 
         'help'  : ('ChartTemplate','Test.stx')},
        )


    #################################################################
    #
    # Initialization code
    #
    #################################################################


	def __init__( self, id, title ):
		""" Create a new ChartTemplate instance """

		RenderableObject.__init__(self, id, klass=Drawing)
		self.title = title
		self._object_counter = 0


    #################################################################
    #
    # Methods required for the object-managerish functionality
    #
    #################################################################


	def manage_delObjects(self, ids=[], REQUEST=None):
		""" """

		rs = RenderableObject.manage_delObjects(self, ids, REQUEST)

		if len(self._objects) == 0:
			self._object_counter = 0

		return rs


	def manage_test(self, REQUEST, RESPONSE, fmt='PNG'):
		"""
	    	put a PNG of the drawing_objects
		"""

		object_test = self.createInstance()
		fmt = lower(fmt)

		if fmt == 'pdf':
			from reportlab.graphics import renderPDF
			content_type = 'application/pdf'
			render_result = renderPDF.drawToString(object_test)
		elif fmt == 'eps':
			from reportlab.graphics import renderPS
			content_type = 'application/postscript'
			render_result = renderPS.drawToString(object_test)
		elif fmt == 'svg':
			from reportlab.graphics import renderSVG
			content_type = 'image/vnd.adobe.svg+xml'
			render_result = renderSVG.drawToString(object_test)
		elif fmt == 'map':
			content_type = None
			from reportlab.graphics import renderIM
			render_result = renderIM.drawToString(object_test)
		elif fmt == 'swf':
			content_type = 'application/x-shockwave-flash'
			from reportlab.graphics import renderSWF
			render_result = renderSWF.drawToString(object_test)
		elif fmt == 'py':
			content_type = 'text/plain'
			render_result = object_test._renderPy()
		else:
			from reportlab.graphics import renderPM
			content_type = 'image/%s' % fmt
			render_result = renderPM.drawToString(object_test, fmt=fmt)

		self.REQUEST.RESPONSE.setHeader('Content-Type', content_type)
		return render_result


def manage_addChartTemplate( self, id, title, sample='', REQUEST=None ):
	""" 
		Called by Zope to create and install a ChartTemplate
	"""

	ct = ChartTemplate( id, title )

	self = self.this()
	self._setObject( id, ct )
 
 	ct = self[id]

	if reportlab_graphics_samples_installed:
	 	if sample == 'bubble':
 			attr = Bubble()
	 	elif sample == 'clustered_bar':
 			attr = ClusteredBar()
	 	elif sample == 'clustered_column':
 			attr = ClusteredColumn()
	 	elif sample == 'exploded_pie':
 			attr = ExplodedPie()
	 	elif sample == 'filled_radar':
 			attr = FilledRadarChart()
	 	elif sample == 'line_chart':
 			attr = LineChart()
	 	elif sample == 'linechart_with_markers':
 			attr = LineChartWithMarkers()
	 	elif sample == 'scatter_lines_markers':
 			attr = ScatterLinesMarkers()
	 	elif sample == 'scatter_lines':
 			attr = ScatterLines()
	 	elif sample == 'scatter':
 			attr = Scatter()
	 	elif sample == 'simple_pie':
 			attr = SimplePie()
	 	elif sample == 'stacked_bar':
 			attr = StackedBar()
	 	elif sample == 'stacked_column':
 			attr = StackedColumn()
	 	else:
 			attr = None
 	else:
		attr = None
 
 	if attr:
 		for key in ['background','chart','Title','Legend','XLabel','YLabel']:
 			subattr = getattr( attr, key, None )
 			if subattr:
 				manage_add = None
 
 				if subattr.__class__ is VerticalBarChart:
 					manage_add = manage_addVerticalBarChart
 				elif subattr.__class__ is HorizontalBarChart:
 					manage_add = manage_addHorizontalBarChart
 				elif subattr.__class__ is LinePlot:
 					manage_add = manage_addLinePlot
 				elif subattr.__class__ is ScatterPlot:
 					manage_add = manage_addScatterPlot
 				elif subattr.__class__ is SpiderChart:
 					manage_add = manage_addSpiderChart
 				elif subattr.__class__ is Legend:
 					manage_add = manage_addLegend
 				elif subattr.__class__ is Label:
 					manage_add = manage_addLabel
 				elif subattr.__class__ is Pie:
 					manage_add = manage_addPie
 
 				if manage_add is not None:
 					manage_add ( ct, id=key, attr=subattr )
 
 	# return to the parent object's manage_main
  	if REQUEST:
  		url = REQUEST['URL1']
		if REQUEST.has_key('add_and_edit'):
			url = ct.absolute_url()
  		REQUEST.RESPONSE.redirect( '%s/manage_main' % url )
 	else:
 		return ct
  
  
InitializeClass( ChartTemplate )


# vim:ts=4
